/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2014 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.annotation.editingsupport;

import java.util.Collection;

import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.ColumnViewer;
import org.eclipse.jface.viewers.EditingSupport;
import org.eclipse.swt.widgets.Composite;
import org.fortiss.tooling.base.model.element.IAnnotatedSpecification;
import org.fortiss.tooling.base.ui.annotation.AnnotationEntry;
import org.fortiss.tooling.base.ui.annotation.valueprovider.EStructuralFeatureDescriptor;
import org.fortiss.tooling.kernel.extension.data.ITopLevelElement;
import org.fortiss.tooling.kernel.service.IPersistencyService;

/**
 * {@link EditingSupport} that supports the selection of multiple values.
 * 
 * @author barner
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating RED Hash: F7D5CB86794EB52A041ABC1398FA7338
 * 
 */
public class MultiSelectionEditingSupport extends AnnotationEditingSupportBase {

	/** The dialog used to select multiple elements. */
	private MultiSelectionCellEditor multiSelectionEditDialog;

	/** The underlying annotation */
	IAnnotatedSpecification specification;

	/** Specifications {@link EStructuralFeatureDescriptor}. */
	private EStructuralFeatureDescriptor eStructuralFeatureDescriptor;

	/** Constructor */
	public MultiSelectionEditingSupport(ColumnViewer viewer,
			Class<? extends IAnnotatedSpecification> clazz, IAnnotatedSpecification specification,
			String instanceKey, EStructuralFeatureDescriptor eStructuralFeatureDescriptor) {
		super(viewer, clazz, instanceKey);

		this.specification = specification;
		this.eStructuralFeatureDescriptor = eStructuralFeatureDescriptor;
		multiSelectionEditDialog =
				new MultiSelectionCellEditor((Composite)getViewer().getControl());
	}

	/** {@inheritDoc} */
	@Override
	protected CellEditor getCellEditor(Object element) {

		if(!(element instanceof AnnotationEntry)) {
			return null;
		}
		AnnotationEntry entry = (AnnotationEntry)element;

		// Only can return a cell editor if the value of the underlying IAnnotatedSpecification is a
		// collection
		Object value = entry.getSpecificationValue(specClass, instanceKey);
		if(!(value instanceof Collection<?>)) {
			return null;
		}

		LabelValueMapping labelValueMapping =
				new LabelValueMapping(eStructuralFeatureDescriptor, specification,
						((AnnotationEntry)element).getModelElement(), null);

		multiSelectionEditDialog.updateData(labelValueMapping, (Collection<?>)value);

		return multiSelectionEditDialog;
	}

	//
	/** {@inheritDoc} */
	@Override
	protected boolean canEdit(Object element) {
		return true;
	}

	//
	/** {@inheritDoc} */
	@Override
	protected Object getValue(Object element) {
		return getLabel(element);
	}

	/**
	 * Returns a {@link String} label for the given annotation (identified by its
	 * {@link AnnotationEntry}).
	 */
	public String getLabel(Object element) {
		if(element instanceof AnnotationEntry) {
			AnnotationEntry entry = (AnnotationEntry)element;

			Object value = entry.getSpecificationValue(specClass, instanceKey);

			if(value instanceof Collection<?>) {
				return "[" + ((Collection<?>)value).size() + " selected]";
			}

		}

		return "[0 selected]";

	}

	/** {@inheritDoc} */
	@Override
	protected void setValue(Object element, final Object value) {
		// Writes back into the model the collection of selected elements
		if(element instanceof AnnotationEntry && value instanceof Collection<?>) {
			final AnnotationEntry entry = (AnnotationEntry)element;

			ITopLevelElement modelContext =
					IPersistencyService.INSTANCE.getTopLevelElementFor(entry.getModelElement());
			modelContext.runAsCommand(new Runnable() {

				@Override
				public void run() {
					try {
						entry.setSpecificationValue(value, specClass);
					} catch(Exception e) {
						// Ignore
					}
				}
			});
		}
	}
}
